@@ -6,6 +6,7 @@
 
 #include "breezehelper.h"
 
+#include "breeze.h"
 #include "breezestyleconfigdata.h"
 
 #include <KColorUtils>
@@ -36,6 +37,8 @@ namespace Breeze
     //* contrast for arrow and treeline rendering
     static const qreal arrowShade = 0.15;
 
+    static const auto radioCheckSunkenDarkeningFactor = 110;
+
     //____________________________________________________________________
     Helper::Helper( KSharedConfig::Ptr config, QObject *parent ) :
         QObject ( parent ),
@@ -93,6 +96,13 @@ namespace Breeze
         _inactiveTitleBarTextColor = appGroup.readEntry( "inactiveForeground", globalGroup.readEntry( "inactiveForeground", palette.color( QPalette::Disabled, QPalette::HighlightedText ) ) );
     }
 
+    QColor transparentize(const QColor& color, qreal amount)
+    {
+        auto clone = color;
+        clone.setAlphaF(amount);
+        return clone;
+    }
+
     //____________________________________________________________________
     QColor Helper::frameOutlineColor( const QPalette& palette, bool mouseOver, bool hasFocus, qreal opacity, AnimationMode mode ) const
     {
@@ -215,121 +225,6 @@ namespace Breeze
 
     }
 
-    //____________________________________________________________________
-    QColor Helper::buttonOutlineColor( const QPalette& palette, bool mouseOver, bool hasFocus, qreal opacity, AnimationMode mode ) const
-    {
-
-        QColor outline( KColorUtils::mix( palette.color( QPalette::Button ), palette.color( QPalette::ButtonText ), 0.3 ) );
-        if( mode == AnimationHover )
-        {
-
-            if( hasFocus )
-            {
-                const QColor focus( buttonFocusOutlineColor( palette ) );
-                const QColor hover( buttonHoverOutlineColor( palette ) );
-                outline = KColorUtils::mix( focus, hover, opacity );
-
-            } else {
-
-                const QColor hover( buttonHoverColor( palette ) );
-                outline = KColorUtils::mix( outline, hover, opacity );
-
-            }
-
-        } else if( mouseOver ) {
-
-            if( hasFocus ) outline = buttonHoverOutlineColor( palette );
-            else outline = buttonHoverColor( palette );
-
-        } else if( mode == AnimationFocus ) {
-
-            const QColor focus( buttonFocusOutlineColor( palette ) );
-            outline = KColorUtils::mix( outline, focus, opacity );
-
-        } else if( hasFocus ) {
-
-            outline = buttonFocusOutlineColor( palette );
-
-        }
-
-        return outline;
-
-    }
-
-    //____________________________________________________________________
-    QColor Helper::buttonBackgroundColor( const QPalette& palette, bool mouseOver, bool hasFocus, bool sunken, qreal opacity, AnimationMode mode ) const
-    {
-
-        QColor background( sunken ?
-            KColorUtils::mix( palette.color( QPalette::Button ), palette.color( QPalette::ButtonText ), 0.2 ):
-            palette.color( QPalette::Button ) );
-
-        if( mode == AnimationHover )
-        {
-
-            const QColor focus( buttonFocusColor( palette ) );
-            const QColor hover( buttonHoverColor( palette ) );
-            if( hasFocus ) background = KColorUtils::mix( focus, hover, opacity );
-
-        } else if( mouseOver && hasFocus ) {
-
-            background = buttonHoverColor( palette );
-
-        } else if( mode == AnimationFocus ) {
-
-            const QColor focus( buttonFocusColor( palette ) );
-            background = KColorUtils::mix( background, focus, opacity );
-
-        } else if( hasFocus ) {
-
-            background = buttonFocusColor( palette );
-
-        }
-
-        return background;
-
-    }
-
-    //____________________________________________________________________
-    QColor Helper::toolButtonColor( const QPalette& palette, bool mouseOver, bool hasFocus, bool sunken, qreal opacity, AnimationMode mode ) const
-    {
-
-        QColor outline;
-        const QColor hoverColor( buttonHoverColor( palette ) );
-        const QColor focusColor( buttonFocusColor( palette ) );
-        const QColor sunkenColor = alphaColor( palette.color( QPalette::WindowText ), 0.2 );
-
-        // hover takes precedence over focus
-        if( mode == AnimationHover )
-        {
-
-            if( hasFocus ) outline = KColorUtils::mix( focusColor, hoverColor, opacity );
-            else if( sunken ) outline = sunkenColor;
-            else outline = alphaColor( hoverColor, opacity );
-
-        } else if( mouseOver ) {
-
-            outline = hoverColor;
-
-        } else if( mode == AnimationFocus ) {
-
-            if( sunken ) outline = KColorUtils::mix( sunkenColor, focusColor, opacity );
-            else outline = alphaColor( focusColor, opacity );
-
-        } else if( hasFocus ) {
-
-            outline = focusColor;
-
-        } else if( sunken ) {
-
-            outline = sunkenColor;
-
-        }
-
-        return outline;
-
-    }
-
     //____________________________________________________________________
     QColor Helper::sliderOutlineColor( const QPalette& palette, bool mouseOver, bool hasFocus, qreal opacity, AnimationMode mode ) const
     {
@@ -521,7 +416,7 @@ namespace Breeze
         painter->setRenderHint( QPainter::Antialiasing );
 
         QRectF frameRect( rect.adjusted( 1, 1, -1, -1 ) );
-        qreal radius( frameRadius( PenWidth::NoPen, -1 ) );
+        qreal radius( frameRadius( PenWidth::NoPen ) );
 
         // set pen
         if( outline.isValid() )
@@ -582,7 +477,7 @@ namespace Breeze
 
             case AllSides:
             {
-                const qreal radius( frameRadius( PenWidth::Frame, -1 ) );
+                const qreal radius( frameRadius( PenWidth::Frame ) );
                 painter->drawRoundedRect( frameRect, radius, radius );
                 break;
             }
@@ -655,95 +550,115 @@ namespace Breeze
     }
 
     //______________________________________________________________________________
-    void Helper::renderButtonFrame(
-        QPainter* painter, const QRect& rect,
-        const QColor& color, const QColor& outline, const QColor& shadow,
-        bool hasFocus, bool sunken ) const
+    void Helper::renderButtonFrame(QPainter* painter, const QRect& rect, const QPalette& palette, const QHash<QByteArray, bool>& stateProperties, qreal bgAnimation, qreal penAnimation) const
     {
+        bool enabled = stateProperties.value("enabled", true);
+        bool visualFocus = stateProperties.value("visualFocus");
+        bool hovered = stateProperties.value("hovered");
+        bool down = stateProperties.value("down");
+        bool checked = stateProperties.value("checked");
+        bool flat = stateProperties.value("flat");
+        bool defaultButton = stateProperties.value("defaultButton");
+        bool hasNeutralHighlight = stateProperties.value("hasNeutralHighlight");
 
-        // setup painter
+        // don't render background if flat and not hovered, down, checked, or given visual focus
+        if (flat && !(hovered || down || checked || visualFocus)
+            && bgAnimation == AnimationData::OpacityInvalid
+            && penAnimation == AnimationData::OpacityInvalid
+        ) {
+            return;
+        }
+
+        QRectF shadowedRect = this->shadowedRect(rect);
+        QRectF frameRect = strokedRect(shadowedRect);
+        qreal radius = frameRadius( PenWidth::Frame );
+        // setting color group to work around KColorScheme feature
+        const QColor &highlightColor = palette.color(!enabled ? QPalette::Disabled : QPalette::Active, QPalette::Highlight);
+        QBrush bgBrush;
+        QBrush penBrush;
+
+        // Colors
+        if (flat) {
+            if (down && enabled) {
+                bgBrush = alphaColor(highlightColor, 0.333);
+            } else if (checked) {
+                bgBrush = hasNeutralHighlight ? alphaColor(neutralText(palette), 0.333)
+                    : alphaColor(palette.buttonText().color(), 0.125);
+                penBrush = hasNeutralHighlight ? neutralText(palette)
+                    : KColorUtils::mix(palette.button().color(), palette.buttonText().color(), 0.3);
+            } else if (defaultButton) {
+                bgBrush = alphaColor(highlightColor, 0.125);
+                penBrush = KColorUtils::mix(
+                    highlightColor,
+                    KColorUtils::mix(palette.button().color(), palette.buttonText().color(), 0.333),
+                    0.5
+                );
+            } else {
+                bgBrush = alphaColor(highlightColor, 0);
+                penBrush = hasNeutralHighlight ? neutralText(palette) : bgBrush;
+            }
+        } else {
+            if (down && enabled) {
+                bgBrush = KColorUtils::mix(palette.button().color(), highlightColor, 0.333);
+            } else if (checked) {
+                bgBrush = hasNeutralHighlight ? KColorUtils::mix(palette.button().color(), neutralText(palette), 0.333)
+                    : KColorUtils::mix(palette.button().color(), palette.buttonText().color(), 0.125);
+                penBrush = hasNeutralHighlight ? neutralText(palette)
+                    : KColorUtils::mix(palette.button().color(), palette.buttonText().color(), 0.3);
+            } else if (defaultButton) {
+                bgBrush = KColorUtils::mix(palette.button().color(), highlightColor, 0.2);
+                penBrush = KColorUtils::mix(
+                    highlightColor,
+                    KColorUtils::mix(palette.button().color(), palette.buttonText().color(), 0.333),
+                    0.5
+                );
+            } else {
+                bgBrush = palette.button().color();
+                penBrush = hasNeutralHighlight ? neutralText(palette)
+                    : KColorUtils::mix(palette.button().color(), palette.buttonText().color(), 0.3);
+            }
+        }
+
+        if ((hovered || visualFocus || down) && enabled) {
+            penBrush = highlightColor;
+        }
+
+        // Animations
+        if (bgAnimation != AnimationData::OpacityInvalid && enabled) {
+            QColor color1 = bgBrush.color();
+            QColor color2 = flat ? alphaColor(highlightColor, 0.333)
+                : KColorUtils::mix(palette.button().color(), highlightColor, 0.333);
+            bgBrush = KColorUtils::mix(color1, color2, bgAnimation);
+        }
+        if (penAnimation != AnimationData::OpacityInvalid && enabled) {
+            QColor color1 = penBrush.color();
+            QColor color2 = highlightColor;
+            penBrush = KColorUtils::mix(color1, color2, penAnimation);
+        }
+
+        // Gradient
+        if (!(flat || down || hovered || checked) && enabled) {
+            QLinearGradient bgGradient(frameRect.topLeft(), frameRect.bottomLeft());
+            bgGradient.setColorAt(0, KColorUtils::mix(bgBrush.color(), Qt::white, 0.03125));
+            bgGradient.setColorAt(0.5, bgBrush.color());
+            bgGradient.setColorAt(1, KColorUtils::mix(bgBrush.color(), Qt::black, 0.03125));
+            QLinearGradient penGradient(frameRect.topLeft(), frameRect.bottomLeft());
+            penGradient.setColorAt(0, KColorUtils::mix(penBrush.color(), Qt::white, 0.03125));
+            penGradient.setColorAt(1, KColorUtils::mix(penBrush.color(), Qt::black, 0.0625));
+            bgBrush = bgGradient;
+            penBrush = penGradient;
+        }
+
+        // Shadow
+        if (!(flat || down || checked) && enabled) {
+            renderRoundedRectShadow(painter, shadowedRect, shadowColor(palette));
+        }
+
+        // Render button
         painter->setRenderHint( QPainter::Antialiasing, true );
-
-        // copy rect
-        QRectF frameRect( rect );
-        frameRect.adjust( 1, 1, -1, -1 );
-        qreal radius( frameRadius( PenWidth::NoPen, -1 ) );
-
-        // shadow
-        if( sunken ) {
-
-            frameRect.translate( 1, 1 );
-
-        } else {
-
-            renderRoundedRectShadow( painter, frameRect, shadow, radius );
-
-        }
-
-        if( outline.isValid() )
-        {
-
-            QLinearGradient gradient( frameRect.topLeft(), frameRect.bottomLeft() );
-            gradient.setColorAt( 0, outline.lighter( hasFocus ? 103:101 ) );
-            gradient.setColorAt( 1, outline.darker( hasFocus ? 110:103 ) );
-            painter->setPen( QPen( QBrush( gradient ), 1.0 ) );
-
-            frameRect = strokedRect( frameRect );
-            radius = frameRadiusForNewPenWidth( radius, PenWidth::Frame );
-
-        } else painter->setPen( Qt::NoPen );
-
-        // content
-        if( color.isValid() )
-        {
-
-            QLinearGradient gradient( frameRect.topLeft(), frameRect.bottomLeft() );
-            gradient.setColorAt( 0, color.lighter( hasFocus ? 103:101 ) );
-            gradient.setColorAt( 1, color.darker( hasFocus ? 110:103 ) );
-            painter->setBrush( gradient );
-
-        } else painter->setBrush( Qt::NoBrush );
-
-        // render
-        painter->drawRoundedRect( frameRect, radius, radius );
-
-    }
-
-    //______________________________________________________________________________
-    void Helper::renderToolButtonFrame(
-        QPainter* painter, const QRect& rect,
-        const QColor& color, bool sunken ) const
-    {
-
-        // do nothing for invalid color
-        if( !color.isValid() ) return;
-
-        // setup painter
-        painter->setRenderHints( QPainter::Antialiasing );
-
-        const QRectF baseRect( rect.adjusted( 1, 1, -1, -1 ) );
-
-        if( sunken )
-        {
-
-            const qreal radius( frameRadius( PenWidth::NoPen ) );
-
-            painter->setPen( Qt::NoPen );
-            painter->setBrush( color );
-
-            painter->drawRoundedRect( baseRect, radius, radius );
-
-        } else {
-
-            const qreal radius( frameRadius( PenWidth::Frame ) );
-
-            painter->setPen( color );
-            painter->setBrush( Qt::NoBrush );
-            const QRectF outlineRect( strokedRect( baseRect ) );
-            painter->drawRoundedRect( outlineRect, radius, radius );
-
-        }
-
+        painter->setBrush(bgBrush);
+        painter->setPen(QPen(penBrush, PenWidth::Frame));
+        painter->drawRoundedRect(frameRect, radius, radius);
     }
 
     //______________________________________________________________________________
@@ -796,7 +711,7 @@ namespace Breeze
         painter->setRenderHint( QPainter::Antialiasing );
 
         QRectF frameRect( rect.adjusted( 1, 1, -1, -1 ) );
-        qreal radius( frameRadius( PenWidth::NoPen, -1 ) );
+        qreal radius( frameRadius( PenWidth::NoPen ) );
 
         // set pen
         if( outline.isValid() )
@@ -862,30 +777,60 @@ namespace Breeze
     //______________________________________________________________________________
     void Helper::renderCheckBoxBackground(
         QPainter* painter, const QRect& rect,
-        const QColor& color, bool sunken ) const
+        const QPalette& palette,
+        CheckBoxState state, bool neutalHighlight,
+        bool sunken, qreal animation ) const
     {
-
         // setup painter
         painter->setRenderHint( QPainter::Antialiasing, true );
 
-        // copy rect and radius
+        // copy rect
         QRectF frameRect( rect );
-        frameRect.adjust( 3, 3, -3, -3 );
+        frameRect.adjust( 2, 2, -2, -2 );
+        frameRect = strokedRect(frameRect);
 
-        if( sunken ) frameRect.translate(1, 1);
+        auto transparent = neutalHighlight ? neutralText(palette) : palette.highlight().color();
+        transparent.setAlphaF(0.50);
 
-        painter->setPen( Qt::NoPen );
-        painter->setBrush( color );
-        painter->drawRect( frameRect );
+        painter->setPen( transparentize( palette.text().color(), 0.5 ) );
+        if (state == CheckOn || state == CheckPartial) {
+            painter->setPen( neutalHighlight ? neutralText(palette) : palette.highlight().color() );
+        }
+
+        const auto radius = Metrics::CheckBox_Radius;
+
+        switch (state) {
+        case CheckOff:
+            painter->setBrush( palette.base().color().darker(sunken ? radioCheckSunkenDarkeningFactor : 100) );
+            painter->drawRoundedRect( frameRect, radius, radius );
+            break;
+
+        case CheckPartial:
+        case CheckOn:
+            painter->setBrush( transparent );
+            painter->drawRoundedRect( frameRect, radius, radius );
+            break;
+
+        case CheckAnimated:
+            painter->setBrush( palette.base().color().darker(sunken ? radioCheckSunkenDarkeningFactor : 100) );
+            painter->drawRoundedRect( frameRect, radius, radius );
+            painter->setBrush( transparent );
+            painter->setOpacity( animation );
+            painter->drawRoundedRect( frameRect, radius, radius );
+            break;
+        }
 
     }
 
     //______________________________________________________________________________
     void Helper::renderCheckBox(
         QPainter* painter, const QRect& rect,
-        const QColor& color, const QColor& shadow,
-        bool sunken, CheckBoxState state, qreal animation ) const
+        const QPalette& palette, bool mouseOver,
+        CheckBoxState state, CheckBoxState target,
+        bool neutalHighlight, bool sunken,
+        qreal animation, qreal hoverAnimation ) const
     {
+        Q_UNUSED(sunken)
 
         // setup painter
         painter->setRenderHint( QPainter::Antialiasing, true );
@@ -893,102 +838,88 @@ namespace Breeze
         // copy rect and radius
         QRectF frameRect( rect );
         frameRect.adjust( 2, 2, -2, -2 );
-        qreal radius( frameRadius( PenWidth::NoPen, -1 ) );
 
-        // shadow
-        if( sunken )
-        {
+        if ( mouseOver ) {
+            painter->save();
 
-            frameRect.translate(1, 1);
+            if (hoverAnimation != AnimationData::OpacityInvalid) {
+                painter->setOpacity(hoverAnimation);
+            }
 
-        } else {
-
-            renderRoundedRectShadow( painter, frameRect, shadow, radius );
-
-        }
-
-        // content
-        {
-
-            painter->setPen( QPen( color, PenWidth::Frame ) );
+            painter->setPen( QPen( neutalHighlight ? neutralText(palette).lighter() : focusColor(palette), PenWidth::Frame ) );
             painter->setBrush( Qt::NoBrush );
 
-            radius = frameRadiusForNewPenWidth( radius, PenWidth::Frame );
-            const QRectF contentRect( strokedRect( frameRect ) );
-            painter->drawRoundedRect( contentRect, radius, radius );
+            painter->drawRoundedRect( frameRect.adjusted(0.5, 0.5, -0.5, -0.5), Metrics::CheckBox_Radius, Metrics::CheckBox_Radius );
 
+            painter->restore();
         }
 
-        // mark
-        if( state == CheckOn )
-        {
+        // check
+        auto leftPoint = frameRect.center();
+        leftPoint.setX(frameRect.left()+4);
 
-            painter->setBrush( color );
-            painter->setPen( Qt::NoPen );
+        auto bottomPoint = frameRect.center();
+        bottomPoint.setX(bottomPoint.x() - 1);
+        bottomPoint.setY(frameRect.bottom() - 5);
 
-            const QRectF markerRect( frameRect.adjusted( 3, 3, -3, -3 ) );
-            painter->drawRect( markerRect );
+        auto rightPoint = frameRect.center();
+        rightPoint.setX(rightPoint.x() + 4.5);
+        rightPoint.setY(frameRect.top() + 5.5);
 
-        } else if( state == CheckPartial ) {
+        QPainterPath path;
+        path.moveTo(leftPoint);
+        path.lineTo(bottomPoint);
+        path.lineTo(rightPoint);
 
-            QPen pen( color, 2 );
-            pen.setJoinStyle( Qt::MiterJoin );
-            painter->setPen( pen );
+        // dots
+        auto centerDot = QRectF(frameRect.center(), QSize(2, 2));
+        centerDot.adjust(-1, -1, -1, -1);
+        auto leftDot = centerDot.adjusted(-4, 0, -4, 0);
+        auto rightDot = centerDot.adjusted(4, 0, 4, 0);
 
-            const QRectF markerRect( frameRect.adjusted( 4, 4, -4, -4 ) );
-            painter->drawRect( markerRect );
+        painter->setPen(Qt::transparent);
+        painter->setBrush(Qt::transparent);
 
-            painter->setPen( Qt::NoPen );
-            painter->setBrush( color );
-            painter->setRenderHint( QPainter::Antialiasing, false );
+        auto checkPen = QPen( palette.text(), PenWidth::Frame * 2 );
+        checkPen.setJoinStyle(Qt::MiterJoin);
 
-            QPainterPath path;
-            path.moveTo( markerRect.topLeft() );
-            path.lineTo( markerRect.right() - 1, markerRect.top() );
-            path.lineTo( markerRect.left(), markerRect.bottom()-1 );
-            painter->drawPath( path );
-
-        } else if( state == CheckAnimated ) {
-
-            const QRectF markerRect( frameRect.adjusted( 3, 3, -3, -3 ) );
-            QPainterPath path;
-            path.moveTo( markerRect.topRight() );
-            path.lineTo( markerRect.center() + animation*( markerRect.topLeft() - markerRect.center() ) );
-            path.lineTo( markerRect.bottomLeft() );
-            path.lineTo( markerRect.center() + animation*( markerRect.bottomRight() - markerRect.center() ) );
-            path.closeSubpath();
-
-            painter->setBrush( color );
-            painter->setPen( Qt::NoPen );
+        switch (state) {
+        case CheckOff:
+            break;
+        case CheckOn:
+            painter->setPen( checkPen );
             painter->drawPath( path );
+            break;
+        case CheckPartial:
+            painter->setBrush( palette.text() );
+            painter->drawRect(leftDot);
+            painter->drawRect(centerDot);
+            painter->drawRect(rightDot);
+            break;
+        case CheckAnimated:
+            checkPen.setDashPattern({ path.length() * animation, path.length() });
 
+            switch (target) {
+            case CheckOff:
+                break;
+            case CheckOn:
+                painter->setPen( checkPen );
+                painter->drawPath( path );
+                break;
+            case CheckPartial:
+                if (animation >= 3/3) painter->drawRect(rightDot);
+                if (animation >= 2/3) painter->drawRect(centerDot);
+                if (animation >= 1/3) painter->drawRect(leftDot);
+                break;
+            case CheckAnimated:
+                break;
+            }
+            break;
         }
-
     }
 
     //______________________________________________________________________________
-    void Helper::renderRadioButtonBackground( QPainter* painter, const QRect& rect, const QColor& color, bool sunken ) const
-    {
-
-        // setup painter
-        painter->setRenderHint( QPainter::Antialiasing, true );
-
-        // copy rect
-        QRectF frameRect( rect );
-        frameRect.adjust( 3, 3, -3, -3 );
-        if( sunken ) frameRect.translate(1, 1);
-
-        painter->setPen( Qt::NoPen );
-        painter->setBrush( color );
-        painter->drawEllipse( frameRect );
-
-    }
-
-    //______________________________________________________________________________
-    void Helper::renderRadioButton(
-        QPainter* painter, const QRect& rect,
-        const QColor& color, const QColor& shadow,
-        bool sunken, RadioButtonState state, qreal animation ) const
+    void Helper::renderRadioButtonBackground( QPainter* painter, const QRect& rect, const QPalette& palette, RadioButtonState state, bool neutalHighlight, bool sunken, qreal animation ) const
     {
 
         // setup painter
@@ -997,55 +928,89 @@ namespace Breeze
         // copy rect
         QRectF frameRect( rect );
         frameRect.adjust( 2, 2, -2, -2 );
+        frameRect.adjust( 0.5, 0.5, -0.5, -0.5 );
 
-        // shadow
-        if( sunken )
-        {
-
-            frameRect.translate( 1, 1 );
-
-        } else {
-
-            renderEllipseShadow( painter, frameRect, shadow );
+        auto transparent = neutalHighlight ? neutralText(palette) : palette.highlight().color();
+        transparent.setAlphaF(0.50);
 
+        painter->setPen( QPen(transparentize( palette.text().color(), 0.5 ), PenWidth::Frame) );
+        if (state == RadioOn) {
+            painter->setPen( QPen(neutalHighlight ? neutralText(palette) : palette.highlight().color(), PenWidth::Frame) );
         }
 
-        // content
-        {
+        switch (state) {
+        case RadioOff:
+            painter->setBrush( palette.base().color().darker(sunken ? radioCheckSunkenDarkeningFactor : 100) );
+            painter->drawEllipse( frameRect );
+            break;
+        case RadioOn:
+            painter->setBrush( transparent );
+            painter->drawEllipse( frameRect );
+            break;
+        case RadioAnimated:
+            painter->setBrush( palette.base().color().darker(sunken ? radioCheckSunkenDarkeningFactor : 100) );
+            painter->drawEllipse( frameRect );
+            painter->setBrush( transparent );
+            painter->setOpacity( animation );
+            painter->drawEllipse( frameRect );
+            break;
+        }
+    }
 
-            painter->setPen( QPen( color, PenWidth::Frame ) );
+    //______________________________________________________________________________
+    void Helper::renderRadioButton(
+        QPainter* painter, const QRect& rect,
+        const QPalette& palette, bool mouseOver,
+        RadioButtonState state, bool neutralHighlight, bool sunken, qreal animation, qreal animationHover ) const
+    {
+        Q_UNUSED(sunken)
+
+        // setup painter
+        painter->setRenderHint( QPainter::Antialiasing, true );
+
+        // copy rect
+        QRectF frameRect( rect );
+        frameRect.adjust( 1, 1, -1, -1 );
+
+        if ( mouseOver ) {
+            painter->save();
+
+            if (animationHover != AnimationData::OpacityInvalid) {
+                painter->setOpacity(animationHover);
+            }
+
+            painter->setPen( QPen( neutralHighlight ? neutralText(palette).lighter() : focusColor(palette), PenWidth::Frame ) );
             painter->setBrush( Qt::NoBrush );
 
-            const QRectF contentRect( strokedRect( frameRect ) );
+            const QRectF contentRect( frameRect.adjusted( 1, 1, -1, -1 ).adjusted( 0.5 , 0.5, -0.5, -0.5 ) );
             painter->drawEllipse( contentRect );
 
+            painter->restore();
         }
 
+        painter->setBrush( palette.text() );
+        painter->setPen( Qt::NoPen );
+
+        QRectF markerRect;
+        markerRect = frameRect.adjusted( 6, 6, -6, -6 );
+        
+        qreal adjustFactor;
+
         // mark
-        if( state == RadioOn )
-        {
-
-            painter->setBrush( color );
-            painter->setPen( Qt::NoPen );
-
-            const QRectF markerRect( frameRect.adjusted( 3, 3, -3, -3 ) );
+        switch (state) {
+        case RadioOn:
             painter->drawEllipse( markerRect );
 
-        } else if( state == RadioAnimated ) {
-
-            painter->setBrush( color );
-            painter->setPen( Qt::NoPen );
-            QRectF markerRect( frameRect.adjusted( 3, 3, -3, -3 ) );
-
-            painter->translate( markerRect.center() );
-            painter->rotate( 45 );
-
-            markerRect.setWidth( markerRect.width()*animation );
-            markerRect.translate( -markerRect.center() );
+            break;
+        case RadioAnimated:
+            adjustFactor = markerRect.height() * (1 - animation);
+            markerRect.adjust(adjustFactor, adjustFactor, -adjustFactor, -adjustFactor);
             painter->drawEllipse( markerRect );
 
+            break;
+        default:
+            break;
         }
-
     }
 
     //______________________________________________________________________________
@@ -1057,14 +1022,17 @@ namespace Breeze
         // setup painter
         painter->setRenderHint( QPainter::Antialiasing, true );
 
-        const QRectF baseRect( rect );
+        QRectF baseRect( rect );
+        baseRect.adjust(0.5, 0.5, -0.5, -0.5);
         const qreal radius( 0.5*Metrics::Slider_GrooveThickness );
 
         // content
         if( color.isValid() )
         {
-            painter->setPen( Qt::NoPen );
-            painter->setBrush( color );
+            painter->setPen( QPen(color, PenWidth::Frame) );
+            auto bg = color;
+            bg.setAlphaF(bg.alphaF() / 2);
+            painter->setBrush( bg );
             painter->drawRoundedRect( baseRect, radius, radius );
         }
 
@@ -1074,7 +1042,7 @@ namespace Breeze
     //______________________________________________________________________________
     void Helper::renderDialGroove(
         QPainter* painter, const QRect& rect,
-        const QColor& color,
+        const QColor& fg, const QColor& bg,
         qreal first, qreal last ) const
     {
 
@@ -1084,7 +1052,7 @@ namespace Breeze
         const QRectF baseRect( rect );
 
         // content
-        if( color.isValid() )
+        if( fg.isValid() )
         {
             const qreal penWidth( Metrics::Slider_GrooveThickness );
             const QRectF grooveRect( rect.adjusted( penWidth/2, penWidth/2, -penWidth/2, -penWidth/2 ) );
@@ -1092,58 +1060,20 @@ namespace Breeze
             // setup angles
             const int angleStart( first * 180 * 16 / M_PI );
             const int angleSpan( (last - first ) * 180 * 16 / M_PI );
+
+            const QPen bgPen( fg, penWidth, Qt::SolidLine, Qt::RoundCap );
+            const QPen fgPen( KColorUtils::overlayColors( bg, alphaColor( fg, 0.5) ), penWidth - 2, Qt::SolidLine, Qt::RoundCap );
             
             // setup pen
             if( angleSpan != 0 )
             {
-                QPen pen( color, penWidth );
-                pen.setCapStyle( Qt::RoundCap );
-                painter->setPen( pen );
+                painter->setPen( bgPen );
                 painter->setBrush( Qt::NoBrush );
                 painter->drawArc( grooveRect, angleStart, angleSpan );
+                painter->setPen( fgPen );
+                painter->drawArc( grooveRect, angleStart, angleSpan );
             }
         }
-
-        
-    }
-
-    //______________________________________________________________________________
-    void Helper::renderDialContents(
-        QPainter* painter, const QRect& rect,
-        const QColor& color,
-        qreal first, qreal second ) const
-    {
-
-        // setup painter
-        painter->setRenderHint( QPainter::Antialiasing, true );
-
-        const QRectF baseRect( rect );
-
-        // content
-        if( color.isValid() )
-        {
-
-            // setup groove rect
-            const qreal penWidth( Metrics::Slider_GrooveThickness );
-            const QRectF grooveRect( rect.adjusted( penWidth/2, penWidth/2, -penWidth/2, -penWidth/2 ) );
-
-            // setup angles
-            const int angleStart( first * 180 * 16 / M_PI );
-            const int angleSpan( (second - first ) * 180 * 16 / M_PI );
-
-            // setup pen
-            if( angleSpan != 0 )
-            {
-                QPen pen( color, penWidth );
-                pen.setCapStyle( Qt::RoundCap );
-                painter->setPen( pen );
-                painter->setBrush( Qt::NoBrush );
-                painter->drawArc( grooveRect, angleStart, angleSpan );
-            }
-
-        }
-
-        
     }
 
     //______________________________________________________________________________
@@ -1174,7 +1104,7 @@ namespace Breeze
         if( outline.isValid() )
         {
 
-            painter->setPen( outline );
+            painter->setPen( QPen(outline, PenWidth::Frame) );
             frameRect = strokedRect( frameRect );
 
         } else painter->setPen( Qt::NoPen );
@@ -1191,20 +1121,21 @@ namespace Breeze
     //______________________________________________________________________________
     void Helper::renderProgressBarGroove(
         QPainter* painter, const QRect& rect,
-        const QColor& color ) const
+        const QColor& fg, const QColor& bg ) const
     {
 
         // setup painter
         painter->setRenderHint( QPainter::Antialiasing, true );
 
-        const QRectF baseRect( rect );
+        QRectF baseRect( rect );
+        baseRect.adjust(0.5,0.5,-0.5,-0.5);
         const qreal radius( 0.5*Metrics::ProgressBar_Thickness );
 
         // content
-        if( color.isValid() )
+        if( fg.isValid() )
         {
-            painter->setPen( Qt::NoPen );
-            painter->setBrush( color );
+            painter->setPen( QPen(fg, PenWidth::Frame) );
+            painter->setBrush( KColorUtils::overlayColors(bg, alphaColor(fg, 0.5)) );
             painter->drawRoundedRect( baseRect, radius, radius );
         }
 
@@ -1269,6 +1200,25 @@ namespace Breeze
 
     //______________________________________________________________________________
     void Helper::renderScrollBarHandle(
+        QPainter* painter, const QRect& rect,
+        const QColor& fg, const QColor& bg ) const
+    {
+
+        // setup painter
+        painter->setRenderHint( QPainter::Antialiasing, true );
+
+        const QRectF baseRect( rect );
+        const qreal radius( 0.5 * std::min({baseRect.width(), baseRect.height(), (qreal)Metrics::ScrollBar_SliderWidth}) );
+
+        painter->setPen( Qt::NoPen );
+        painter->setPen( QPen(fg, 1.001) );
+        painter->setBrush( KColorUtils::overlayColors(bg, alphaColor(fg, 0.5)) );
+        painter->drawRoundedRect( strokedRect(baseRect), radius, radius );
+
+        
+    }
+
+    void Helper::renderScrollBarGroove(
         QPainter* painter, const QRect& rect,
         const QColor& color ) const
     {
@@ -1283,8 +1233,11 @@ namespace Breeze
         if( color.isValid() )
         {
             painter->setPen( Qt::NoPen );
-            painter->setBrush( color );
-            painter->drawRoundedRect( baseRect, radius, radius );
+            auto bg = color;
+            bg.setAlphaF(bg.alphaF() / 2.0);
+            painter->setBrush( bg );
+            painter->setPen( QPen(color, 1.001) );
+            painter->drawRoundedRect( strokedRect(baseRect), radius, radius );
         }
 
         
@@ -1318,7 +1271,7 @@ namespace Breeze
         painter->setRenderHint( QPainter::Antialiasing, true );
 
         QRectF frameRect( rect );
-        qreal radius( frameRadius( PenWidth::NoPen, -1 ) );
+        qreal radius( frameRadius( PenWidth::NoPen ) );
 
         // pen
         if( outline.isValid() )
@@ -1472,36 +1425,22 @@ namespace Breeze
         painter->restore();
 
     }
-    
+
     //______________________________________________________________________________
     void Helper::renderRoundedRectShadow( QPainter* painter, const QRectF& rect, const QColor& color, qreal radius ) const
     {
         if( !color.isValid() ) return;
-        
-        painter->save();
-        
-        qreal translation = 0.5 * PenWidth::Shadow; // Translate for the pen
-        
-        /* Clipping prevents shadows from being visible inside checkboxes.
-         * Clipping away unneeded parts here also improves performance by 40-60%
-         * versus using just an outline of a rectangle.
-         * Tested by looking at the paint analyser in GammaRay.
-         */
-        // Right side
-        QRegion clip( rect.right() - std::ceil( radius ), rect.top(), 
-                      std::ceil( radius ) + PenWidth::Shadow, rect.height() );
-        // Bottom side
-        clip = clip.united( QRegion( rect.left(), rect.bottom() - std::ceil( radius ), 
-                                     rect.width(), std::ceil( radius ) + PenWidth::Shadow ) );
 
-        painter->setClipRegion( clip );
-        painter->setPen( color );
+        painter->setRenderHint( QPainter::Antialiasing, true );
+
+        qreal adjustment = 0.5 * PenWidth::Shadow; // Translate for the pen
+        QRectF shadowRect = rect.adjusted( adjustment, adjustment, -adjustment, adjustment );
+
+        painter->setPen( QPen(color, PenWidth::Shadow) );
         painter->setBrush( Qt::NoBrush );
-        painter->drawRoundedRect( rect.translated( translation, translation ), radius, radius );
-        
-        painter->restore();
+        painter->drawRoundedRect( shadowRect, radius, radius );
     }
-    
+
     //______________________________________________________________________________
     void Helper::renderEllipseShadow( QPainter* painter, const QRectF& rect, const QColor& color ) const
     {
@@ -1545,7 +1484,7 @@ namespace Breeze
     }
 
     //______________________________________________________________________________
-    QRectF Helper::strokedRect( const QRectF &rect, const int penWidth ) const
+    QRectF Helper::strokedRect( const QRectF &rect, const qreal penWidth ) const
     {
         /* With a pen stroke width of 1, the rectangle should have each of its
          * sides moved inwards by half a pixel. This allows the stroke to be
@@ -1554,12 +1493,7 @@ namespace Breeze
          * original size of the rectangle.
          */
         qreal adjustment = 0.5 * penWidth;
-        return QRectF( rect ).adjusted( adjustment, adjustment, -adjustment, -adjustment );
-    }
-    
-    QRectF Helper::strokedRect( const QRect &rect, const int penWidth ) const
-    {
-        return strokedRect(QRectF(rect), penWidth);
+        return rect.adjusted( adjustment, adjustment, -adjustment, -adjustment );
     }
 
     //______________________________________________________________________________
